/* MQTTSens1
 *
 *******************************************************************************
 * Project:		MQTTSens1
 *
 * Author:		Walter Trojan
 *
 * Description: Measurement of Temp, Humity an Light
 * 				Sending to MQTT
 *
 * IDE:			Eclipse
 *
 * Finished:	xx.xx.xx
 *
 *  05.09.16	user_main_61.c
 *				bernahme von MQTTBench
 *
 *	07.09.16	user_main_62
 *				Senden von MQTT msg im Abstand von 10s
 *				DHT + ADC Werte
 *
 *	08.09.16	user_main_63
 *				auch DS18B20
 *
 *
 *******************************************************************************/


#include <ets_sys.h>
#include <osapi.h>
#include <os_type.h>
#include <user_interface.h>
#include <espconn.h>
#include "mqtt.h"
#include "debug.h"
#include "mem.h"
#include "uart.h"
#include "uart_register.h"
#include "config.h"
#include "uart.h"
#include "driver/dht22.h"
#include "driver/ds18b20.h"

#define LED_GN_MUX		PERIPHS_IO_MUX_MTDO_U
#define LED_GN_FUNC		FUNC_GPIO15
#define LED_GN_PIN		15

/*******************************************************************************
 * Global variables
 ******************************************************************************/
MQTT_Client mqttClient;
static ETSTimer Wloop;
static ETSTimer Systic;


bool     Gopub;
bool     GoDS;
uint32_t Systim = 0;
uint32_t Pubtim = 0;
bool	 RdyInit;
bool     RdyCon;
bool	 RdyIP;
bool     RdyMQ;
bool	 RdyPub;
bool     RdySub = false;
char     TopBuf[20];
char     DatBuf[20];
uint32_t Benchcount;
char     msg[64];

char 	 temp[10];
char 	 hum[10];
struct   dht_sensor_data* r;
float    lastTemp, lastHum;
uint16_t Adcval;

uint8_t data[12];
uint8_t DScon[5];
uint8_t DSpin[5];
uint8_t DStemp [5];
char    DSsign [5];

const char *FlashSizeMap[] =
{
		"512 KB (256 KB + 256 KB)",	// 0x00
		"256 KB",			// 0x01
		"1024 KB (512 KB + 512 KB)", 	// 0x02
		"2048 KB (512 KB + 512 KB)"	// 0x03
		"4096 KB (512 KB + 512 KB)"	// 0x04
		"2048 KB (1024 KB + 1024 KB)"	// 0x05
		"4096 KB (1024 KB + 1024 KB)"	// 0x06
};

const char *WiFiMode[] =
{
		"NULL",		// 0x00
		"STATION",	// 0x01
		"SOFTAP", 	// 0x02
		"STATIONAP"	// 0x03
};

const char *WiFiStatus[] =
{
	    "STATION_IDLE", 			// 0x00
	    "STATION_CONNECTING", 		// 0x01
	    "STATION_WRONG_PASSWORD", 	// 0x02
	    "STATION_NO_AP_FOUND", 		// 0x03
	    "STATION_CONNECT_FAIL", 	// 0x04
	    "STATION_GOT_IP" 			// 0x05
};


/*******************************************************************************
 * DS_Init: Initialize DS18B20
 ******************************************************************************/
void ICACHE_FLASH_ATTR DS_Init(void)
{
	uint32_t i;

	DS_Gpio();
	DSpin[0] = DSvorl_PIN;
	DSpin[1] = DSback_PIN;
	DScon[0] = DS_reset(DSvorl_PIN);
	//os_delay_us(30000);
	DScon[1] = DS_reset(DSback_PIN);
	INFO("DScon %d %d \n\r", DScon[0],DScon[1]);

	for(i=0; i<2; i++)
	{
		if(DScon[i] == 1)
		{
			os_delay_us(30000);
			DS_reset(DSpin[i]);
			DS_write(DS1820_SKIP_ROM, 0,DSpin[i]);
			DS_write(DS1820_WRITE_SCRATCHPAD,0,DSpin[i]);
			DS_write(0x00,0,DSpin[i]);
			DS_write(0x00,0,DSpin[i]);
			DS_write(0x1F,0,DSpin[i]);
			INFO("DS_Init %d\n\r",i);
		}
	}

}

/*******************************************************************************
 * DS_Start: Start DS18B20 conversion
 ******************************************************************************/
void ICACHE_FLASH_ATTR DS_Start(void)
{
	uint32_t i;

	for(i=0; i<2; i++)
	{
		if(DScon[i] == 1)
		{
			os_delay_us(30000);
			DS_reset(DSpin[i]);
			DS_write(DS1820_SKIP_ROM, 0,DSpin[i]);
			DS_write(DS1820_CONVERT_T,0,DSpin[i]);
		}
	}

}

/*******************************************************************************
 * DS_Temp: Gets DS18B20 temperature
 ******************************************************************************/
void ICACHE_FLASH_ATTR DS_Temp(void)
{
	uint32_t i,j;
	int HighByte, LowByte, TReading, SignBit, Tc_100, Whole, Fract;

	for(i=0; i<2; i++)
	{
		os_delay_us(30000);
		DStemp[i] = 99;
		DSsign[i] = '-';
		if(DScon[i] == 1)
		{
			DS_reset(DSpin[i]);
			DS_write(DS1820_SKIP_ROM,0,DSpin[i]);
			DS_write(DS1820_READ_SCRATCHPAD,0,DSpin[i]);

			for(j = 0; j < 9; j++)
				data[j] = DS_read(DSpin[i]);

			LowByte = data[0];
			HighByte = data[1];
			TReading = (HighByte << 8) + LowByte;
			SignBit = TReading & 0x8000;  			// test most sig bit
			if (SignBit) 							// negative
				TReading = (TReading ^ 0xffff) + 1; // 2's comp

			Whole = TReading >> 4;  // separate off the whole and fractional portions
			Fract = (TReading & 0xf) * 100 / 16;
			DStemp[i] = Whole;
			if(Fract > 0)
				DStemp[i]++;
			if(SignBit)
				DSsign[i] = '-';
			else
				DSsign[i] = '+';

		}
	}
}


/*******************************************************************************
 * WiFi_Connect: Connect station to router
 ******************************************************************************/
void ICACHE_FLASH_ATTR WIFI_Connect(uint8_t* ssid, uint8_t* pass)
{
	struct station_config stationConf;
	bool retval;								//

	INFO("WIFI_INIT\r\n");
	wifi_set_opmode(STATION_MODE);
	wifi_station_disconnect();					//
	wifi_station_dhcpc_stop();					//
	os_memset(&stationConf, 0, sizeof(struct station_config));

	os_sprintf(stationConf.ssid, "%s", ssid);
	//INFO("...SSID=%s\n\r",ssid);				//
	os_sprintf(stationConf.password, "%s", pass);
	//INFO("...PASS=%s\n\r",pass);
	stationConf.bssid_set = 0; 					//need not check MAC address of AP

	retval = wifi_station_set_config(&stationConf);
	if(retval)									//
		INFO("wifi_station_set = OK\n\r");

	retval = wifi_station_connect();
	wifi_station_dhcpc_start();					//
	if(retval)									//
		INFO("wifi_station_connect = OK\n\r");
	else
		INFO("wifi_station_connect = NNNOK\n\r");
}



/*******************************************************************************
 * MQTT callbacks
 ******************************************************************************/
void mqtt_connected_cb(uint32_t *args)
{

	os_delay_us(65000);
	INFO(" ==> MQTT: Connected\r\n");
	MQTT_Client* client = (MQTT_Client*)args;

/*
	TimeSta = system_get_time();
	Benchcount = 1;
	os_sprintf(msg, "\r\nStart \r\n%4u  ESP01 Bench 0123456789012345678901234567890", Benchcount);
	MQTT_Publish(&mqttClient, "ESP01/bench", msg, strlen(msg), 0, 0);

*/
}

void mqtt_disconnected_cb(uint32_t *args)
{
	MQTT_Client* client = (MQTT_Client*)args;
	RdyMQ = false;
	INFO(" ==> MQTT: Disconnected\r\n");
}

void mqtt_published_cb(uint32_t *args)
{
	MQTT_Client* client = (MQTT_Client*)args;
	INFO("MQTT: Published\r\n");
/*
	Benchcount = Benchcount + 1;
	if(Benchcount < 101){
		os_sprintf(msg, "%4u  ESP01 Bench 0123456789012345678901234567890", Benchcount);
		MQTT_Publish(&mqttClient, "ESP01/bench", msg, strlen(msg), 0, 0);
	}

	if(Benchcount == 101){
		TimeEnd = system_get_time();
		TimeGes = TimeEnd - TimeSta;
		os_sprintf(msg, "==> Zeit:%4u uS ", TimeGes);
		MQTT_Publish(&mqttClient, "ESP01/bench", msg, strlen(msg), 0, 0);
	}
*/
}

void mqtt_data_cb(uint32_t *args, const char* topic, uint32_t topic_len, const char *data, uint32_t data_len)
{
	char *topicBuf = (char*)os_zalloc(topic_len+1),
			*dataBuf = (char*)os_zalloc(data_len+1);

	MQTT_Client* client = (MQTT_Client*)args;

	os_memcpy(topicBuf, topic, topic_len);
	topicBuf[topic_len] = 0;

	os_memcpy(dataBuf, data, data_len);
	dataBuf[data_len] = 0;

	INFO("/r/n Receive topic: %s, data: %s \r\n", topicBuf, dataBuf);

	RdySub = true;
	strcpy(TopBuf,topicBuf);
	strcpy(DatBuf,dataBuf);

	os_free(topicBuf);
	os_free(dataBuf);
}


/*******************************************************************************
 * init_done_cb: Called after init is ready
 ******************************************************************************/

static void ICACHE_FLASH_ATTR init_done_cb(void)
{
	os_delay_us(65000);
	INFO("\r\n ==> Init done.\r\n");
	WIFI_Connect(config.sta_ssid, config.sta_pwd);
}


/*******************************************************************************
 * eventHandler: Called after some events
 ******************************************************************************/
static void ICACHE_FLASH_ATTR eventHandler(System_Event_t *event) {
	switch(event->event) {
		case EVENT_STAMODE_CONNECTED:
		{
			INFO(" ==> Event: EVENT_STAMODE_CONNECTED\r\n");
			break;
		}
		case EVENT_STAMODE_DISCONNECTED:
		{
			INFO(" ==> Event: EVENT_STAMODE_DISCONNECTED\r\n");
			break;
		}
		case EVENT_STAMODE_AUTHMODE_CHANGE:
		{
			INFO(" Event: EVENT_STAMODE_AUTHMODE_CHANGE\r\n");
			break;
		}
		case EVENT_STAMODE_GOT_IP:
		{
			INFO(" ==> Event: EVENT_STAMODE_GOT_IP\r\n");
			os_delay_us(65000);
			MQTT_Connect(&mqttClient);
			break;
		}
		case EVENT_SOFTAPMODE_STACONNECTED:
		{
			INFO(" Event: EVENT_SOFTAPMODE_STACONNECTED\r\n");
			break;
		}
		case EVENT_SOFTAPMODE_STADISCONNECTED:
		{
			INFO(" Event: EVENT_SOFTAPMODE_STADISCONNECTED\r\n");
			break;
		}
		default:
		{
			INFO(" Unexpected event: %d\r\n", event->event);
			break;
		}
	}
}





/********************************************************************************
 * sysclk: Will be called cyclic by RTOS, depending on timer Systic (every 10 ms)
 ******************************************************************************/

LOCAL void ICACHE_FLASH_ATTR sysclk(void *arg)
{
	Systim++;
	Pubtim++;
}



/*******************************************************************************
 * workloop: Will be by RTOS, depending on timer Wloop, calling itself, if ready
 ******************************************************************************/

static void ICACHE_FLASH_ATTR workloop(void *arg)
{
	os_timer_disarm(&Wloop);

	if((Systim >= 0) && (Systim <= 5))						// blink green LED
		GPIO_OUTPUT_SET(LED_GN_PIN, 1);

	if((Systim >= 50) && (Systim <= 55))
		GPIO_OUTPUT_SET(LED_GN_PIN, 0);

	if(Systim >= 100)
		Systim = 0;

	if(Pubtim >= 1000){						// 10 sec.
		Pubtim = 0;
		GoDS   = true;
		DS_Start();

		r = DHTRead();
		lastTemp = r->temperature;
		lastHum  = r->humidity;
		if(r->success)
		{
			os_sprintf(temp, "%d.%d",(int)(lastTemp),(int)((lastTemp - (int)lastTemp)*100));
			os_sprintf(hum, "%d.%d",(int)(lastHum),(int)((lastHum - (int)lastHum)*100));
			INFO("Temperature: %s *C, Humidity: %s %%\r\n", temp, hum);
			os_sprintf(msg, "\n\rSens1_Humi=%d.%d",(int)(lastHum),(int)((lastHum - (int)lastHum)*100));
			MQTT_Publish(&mqttClient, "Sens1/humi", msg, strlen(msg), 0, 0);
			os_sprintf(msg, "Sens1_Temp=%d.%d",(int)(lastTemp),(int)((lastTemp - (int)lastTemp)*100));
			MQTT_Publish(&mqttClient, "Sens1/temp", msg, strlen(msg), 0, 0);
		}
		else
			INFO("Error reading temperature and humidity.\r\n");

		Adcval = system_adc_read();
		INFO("ADC-Value = %d\r\n", Adcval);
		os_sprintf(msg, "Sens1_Lght=%d",Adcval);
		MQTT_Publish(&mqttClient, "Sens1/lght", msg, strlen(msg), 0, 0);
	}

	if((GoDS) && (Pubtim >= 20))
	{
		GoDS = false;
		DS_Temp();
		INFO("DS18B20 = %c%2d  %c%2d \r\n",DSsign[0],DStemp[0],DSsign[1],DStemp[1]);
		os_sprintf(msg, "Sens1_DSTemp=%c%2d  %c%2d",DSsign[0],DStemp[0],DSsign[1],DStemp[1]);
		MQTT_Publish(&mqttClient, "Sens1/dstemp", msg, strlen(msg), 0, 0);
	}

	os_timer_setfn(&Wloop, (os_timer_func_t *)workloop, NULL);
	os_timer_arm(&Wloop, 10, 0);

}


/*******************************************************************************
 * user_init: Performs once for initialization
 ******************************************************************************/
void user_init(void)
{


	uart_init(BIT_RATE_115200, BIT_RATE_115200);
	os_delay_us(65000);

	system_set_os_print(1);
	INFO("\r\n\n\n==== System info: ====\n");
	INFO("SDK version:%s rom %d\n", system_get_sdk_version(), system_upgrade_userbin_check());
	INFO("Time = %ld\n", system_get_time());
	INFO("Chip id = 0x%x\n", system_get_chip_id());
	INFO("CPU freq = %d MHz\n", system_get_cpu_freq());
	INFO("Flash size map = %s\n", FlashSizeMap[system_get_flash_size_map()]);
	INFO("Free heap size = %d\n", system_get_free_heap_size());
	INFO("==== End System info ====\n\n");
	os_delay_us(6000);

	PIN_FUNC_SELECT(LED_GN_MUX,  LED_GN_FUNC);
	Systim = 0;
	GPIO_OUTPUT_SET(LED_GN_PIN, 0);

	DHTInit(DHT22);
	DS_Init();
	GoDS   = false;
	Pubtim = 0;

	wifi_station_set_auto_connect(0);				// no auto connection
	system_init_done_cb(init_done_cb);				// register init_done_cb
	wifi_set_event_handler_cb(eventHandler);		// register event handler

	config_load();

	RdyInit = false;
	RdyCon  = false;
	RdyIP   = false;
	RdyMQ   = false;
	RdyPub  = false;


	MQTT_InitConnection(&mqttClient, config.mqtt_host, config.mqtt_port, config.security);
	MQTT_InitClient(&mqttClient, config.device_id, config.mqtt_user, config.mqtt_pass, config.mqtt_keepalive, 1);
	MQTT_InitLWT(&mqttClient, "/lwt", "offline", 0, 0);
	MQTT_OnConnected(&mqttClient, mqtt_connected_cb);
	MQTT_OnDisconnected(&mqttClient, mqtt_disconnected_cb);
	MQTT_OnPublished(&mqttClient, mqtt_published_cb);
	MQTT_OnData(&mqttClient, mqtt_data_cb);



	os_timer_disarm(&Wloop);
	os_timer_setfn(&Wloop, (os_timer_func_t *)workloop, NULL);
	os_timer_arm(&Wloop, 10, 0);

	os_timer_disarm(&Systic);
	os_timer_setfn(&Systic, (os_timer_func_t *)sysclk, NULL);
	os_timer_arm(&Systic, 10, 1);

	INFO("\r\nSystem started ...\r\n");
}

//void user_rf_pre_init(void) {}
